/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL06 for C using RDMSQL */
/* When you run this example, two databases are created each with a table.
 * Each table is populated with a few rows and finally the data read with
 * a single query.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDMSQL call.
 */
static void drop_database(
    HCONN       hDbc,
    const char *name)
{
    HSTMT hStmt;
    char  commandText[80];

    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    sprintf(commandText, "DROP DATABASE %s", name);
    (void) rsqlExecDirect(hStmt, commandText);
    (void) rsqlExecDirect(hStmt, "COMMIT");

    (void) rsqlFreeStmt(hStmt);
}

static void create_this_database(
    HCONN       hDbc,
    const char *name,
    const char *cmds[],
    uint32_t    cmd_length)
{
    RSQL_ERRCODE rsqlCode;
    HSTMT        hStmt;
    char         commandText[80];
    uint16_t     ii;

    (void) rsqlAllocStmt(hDbc, &hStmt);

    sprintf(commandText, "CREATE DATABASE %s", name);
    rsqlCode = rsqlExecDirect(hStmt, commandText);

    for (ii = 0; ii < cmd_length; ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the %s database\n", name);

    (void) rsqlExecDirect(hStmt, "COMMIT");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, name);

    (void) rsqlFreeStmt(hStmt);
}

static void create_databases(
    HCONN hDbc)
{
    const char *cmds1[] = {
        "CREATE TABLE Table1 (id INTEGER PRIMARY KEY, str CHAR(30))"
    };
    const char *cmds2[] = {
        "CREATE TABLE Table2 (id INTEGER, str CHAR(30))"
    };

    /* Drop any existing copies of the databases. */
    drop_database(hDbc, "SQL06_1");
    drop_database(hDbc, "SQL06_2");

    /* Attempt to create the databases */
    create_this_database(hDbc, "SQL06_1", cmds1, 1);
    create_this_database(hDbc, "SQL06_2", cmds2, 1);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc;
    HSTMT             hStmt;
    const RSQL_VALUE *result;
    uint16_t          numCols;
    char              commandText[80];
    int16_t           ii;

    /* Allocate handles */
    (void) rsqlAllocConn(&hDbc);

    create_databases(hDbc);

    /* Specify which databases to use */
    rsqlCode = rsqlOpenDB(hDbc, "SQL06_1;SQL06_2", "x");

    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Insert one row into the table in the first database */
    rsqlCode = rsqlExecDirect(hStmt, "INSERT INTO Table1 VALUES 1, 'database1 1'");

    /* Insert three rows into the table in the second database */
    for (ii = 1; ii < 4; ii++)
    {
        sprintf(commandText, "INSERT INTO Table2 VALUES 1, 'database2 %hd'", ii);
        rsqlCode = rsqlExecDirect(hStmt, commandText);
    }

    /* Scan through the data, generating the result */
    rsqlCode = rsqlExecDirect(hStmt, "SELECT Table1.str, Table2.str FROM Table1, Table2 WHERE Table1.id = Table2.id");

    while ((rsqlCode = rsqlFetch(hStmt, &result, &numCols)) == errSUCCESS)
        printf("%s -- %s\n", result[0].vt.cv, result[1].vt.cv);

    /* Close the database */
    rsqlCode = rsqlCloseDB(hDbc, "SQL06_1;SQL06_2");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

